@@ -1,4 +1,8 @@
 #!/usr/bin/env python2
+# Copyright 2016 The Emscripten Authors.  All rights reserved.
+# Emscripten is available under two separate licenses, the MIT license and the
+# University of Illinois/NCSA Open Source License.  Both these licenses can be
+# found in the LICENSE file.
 
 '''
 emar - ar helper script
@@ -12,8 +16,9 @@ from tools.toolchain_profiler import ToolchainProfiler
 if __name__ == '__main__':
   ToolchainProfiler.record_process_start()
 
-import os, subprocess, sys
+import os, sys
 from tools import shared
+from tools.response_file import substitute_response_files
 
 #
 # Main run() function
@@ -23,6 +28,8 @@ def run():
   if DEBUG == "0":
     DEBUG = None
 
+  sys.argv = substitute_response_files(sys.argv)
+
   newargs = [shared.LLVM_AR] + sys.argv[1:]
 
   if DEBUG:
@@ -60,10 +67,8 @@ def run():
           break
         i += 1
 
-  if DEBUG:
-    print('Invoking ' + str(newargs))
   try:
-    return subprocess.call(newargs, stdin=sys.stdin)
+    return shared.run_process(newargs, stdin=sys.stdin).returncode
   finally:
     for d in to_delete:
       shared.try_delete(d)
