@@ -727,10 +727,10 @@ mergeInto(LibraryManager.library, {
         if (event === 'error') {
           var sp = stackSave();
           var msg = allocate(intArrayFromString(data[2]), 'i8', ALLOC_STACK);
-          Module['dynCall_viiii'](callback, data[0], data[1], msg, userData);
+          {{{ makeDynCall('viiii') }}}(callback, data[0], data[1], msg, userData);
           stackRestore(sp);
         } else {
-          Module['dynCall_vii'](callback, data, userData);
+          {{{ makeDynCall('vii') }}}(callback, data, userData);
         }
       } catch (e) {
         if (e instanceof ExitStatus) {
