@@ -5,11 +5,15 @@
 
 //
 // Various compiling-to-JS parameters. These are simply variables present when the
-// JS compiler runs. To set them, do something like
+// JS compiler runs. To set them, do something like:
 //
 //   emcc -s OPTION1=VALUE1 -s OPTION2=VALUE2 [..other stuff..]
 //
-// See https://github.com/kripken/emscripten/wiki/Code-Generation-Modes/
+// For convenience and readability `-s OPTION` expands to `-s OPTION=1`
+// and `-s NO_OPTION` expands to `-s OPTION=0` (assuming OPTION is a valid
+// option).
+//
+// See https://github.com/emscripten-core/emscripten/wiki/Code-Generation-Modes/
 //
 // Note that the values here are the defaults in -O0, that is, unoptimized
 // mode. See apply_opt_level in tools/shared.py for how -O1,2,3 affect these
@@ -109,6 +113,11 @@ var MALLOC = "dlmalloc";
 // returning NULL (0) when it fails.
 var ABORTING_MALLOC = 1;
 
+// If 1, generated a version of memcpy() and memset() that unroll their
+// copy sizes. If 0, optimizes for size instead to generate a smaller memcpy.
+// This flag only has effect when targeting asm.js.
+var FAST_UNROLLED_MEMCPY_AND_MEMSET = 1;
+
 // If false, we abort with an error if we try to allocate more memory than
 // we can (TOTAL_MEMORY). If true, we will grow the memory arrays at
 // runtime, seamlessly and dynamically. This has a performance cost in asm.js,
@@ -127,6 +136,10 @@ var ABORTING_MALLOC = 1;
 // memory from the system as necessary.
 var ALLOW_MEMORY_GROWTH = 0;
 
+// If true, allows more functions to be added to the table at runtime. This is
+// necessary for dynamic linking, and set automatically in that mode.
+var ALLOW_TABLE_GROWTH = 0;
+
 // where global data begins; the start of static memory. -1 means use the
 // default, any other value will be used as an override
 var GLOBAL_BASE = -1;
@@ -137,9 +150,6 @@ var GLOBAL_BASE = -1;
 // allocations, by forcing the stack to start in the same place their
 // memory usage patterns would be the same.
 
-// Code embetterments
-var STACK_START = -1;
-
 // How to load and store 64-bit doubles.  A potential risk is that doubles may
 // be only 32-bit aligned. Forcing 64-bit alignment in Clang itself should be
 // able to solve that, or as a workaround in DOUBLE_MODE 1 we will carefully
@@ -174,14 +184,6 @@ var FORCE_ALIGNED_MEMORY = 0;
 // wasm.
 var WARN_UNALIGNED = 0;
 
-// If enabled, i64 addition etc. is emulated - which is slow but precise. If
-// disabled, we use the 'double trick' which is fast but incurs rounding at high
-// values.  If set to 2, we always include the i64 math code, which is necessary
-// in the case that we can't know at compile time that 64-bit math is needed.
-// For example, if you print 64-bit values with printf, but never add them, we
-// can't know at compile time and you need to set this to 2.
-var PRECISE_I64_MATH = 1;
-
 // 0: Use JS numbers for floating-point values. These are 64-bit and do not model C++
 //    floats exactly, which are 32-bit.
 // 1: Model C++ floats precisely, using Math.fround, polyfilling when necessary. This
@@ -213,6 +215,18 @@ var SIMD = 0;
 // Whether closure compiling is being run on this output
 var USE_CLOSURE_COMPILER = 0;
 
+// If set to 1, each asm.js/wasm module export is individually declared with a
+// JavaScript "var" definition. This is the simple and recommended approach.
+// However, this does increase code size (especially if you have many such
+// exports), which can be avoided in an unsafe way by setting this to 0. In that
+// case, no "var" is created for each export, and instead a loop (of small
+// constant code size, no matter how many exports you have) writes all the
+// exports received into the global scope. Doing so is dangerous since such
+// modifications of the global scope can confuse external JS minifer tools, and
+// also things can break if the scope the code is in is not the global scope
+// (e.g. if you manually enclose them in a function scope).
+var DECLARE_ASM_MODULE_EXPORTS = 1;
+
 // Ignore closure warnings and errors (like on duplicate definitions)
 var IGNORE_CLOSURE_COMPILER_ERRORS = 0;
 
@@ -357,6 +371,11 @@ var WEBSOCKET_SUBPROTOCOL = 'binary';
 // Print out debugging information from our OpenAL implementation.
 var OPENAL_DEBUG = 0;
 
+// If 1, prints out debugging related to calls from emscripten_web_socket_* functions
+// in emscripten/websocket.h.
+// If 2, additionally traces bytes communicated via the sockets.
+var WEBSOCKET_DEBUG = 0;
+
 // Adds extra checks for error situations in the GL library. Can impact
 // performance.
 var GL_ASSERTIONS = 0;
@@ -381,6 +400,47 @@ var GL_UNSAFE_OPTS = 1;
 // Forces support for all GLES2 features, not just the WebGL-friendly subset.
 var FULL_ES2 = 0;
 
+// If true, glGetString() for GL_VERSION and GL_SHADING_LANGUAGE_VERSION will
+// return strings OpenGL ES format "Open GL ES ... (WebGL ...)" rather than the
+// WebGL format. If false, the direct WebGL format strings are returned. Set
+// this to true to make GL contexts appear like an OpenGL ES context in these
+// version strings (at the expense of a little bit of added code size), and to
+// false to make GL contexts appear like WebGL contexts and to save some bytes
+// from the output.
+var GL_EMULATE_GLES_VERSION_STRING_FORMAT = 1;
+
+// If true, all GL extensions are advertised in both unprefixed WebGL extension
+// format, but also in desktop/mobile GLES/GL extension format with "GL_" prefix.
+var GL_EXTENSIONS_IN_PREFIXED_FORMAT = 1;
+
+// If true, adds support for automatically enabling all GL extensions for
+// GLES/GL emulation purposes. This takes up code size. If you set this to 0,
+// you will need to manually enable the extensions you need.
+var GL_SUPPORT_AUTOMATIC_ENABLE_EXTENSIONS = 1;
+
+// If set to 0, Emscripten GLES2->WebGL translation layer does not track the kind
+// of GL errors that exist in GLES2 but do not exist in WebGL. Settings this to 0
+// saves code size. (Good to keep at 1 for development)
+var GL_TRACK_ERRORS = 1;
+
+// If true, GL contexts support the explicitSwapControl context creation flag.
+// Set to 0 to save a little bit of space on projects that do not need it.
+var GL_SUPPORT_EXPLICIT_SWAP_CONTROL = 0;
+
+// If true, calls to glUniform*fv and glUniformMatrix*fv utilize a pool of
+// preallocated temporary buffers for common small sizes to avoid generating
+// temporary garbage for WebGL 1. Disable this to optimize generated size of the
+// GL library a little bit, at the expense of generating garbage in WebGL 1. If
+// you are only using WebGL 2 and do not support WebGL 1, this is not needed and
+// you can turn it off.
+var GL_POOL_TEMP_BUFFERS = 1;
+
+// Some old Android WeChat (Chromium 37?) browser has a WebGL bug that it ignores
+// the offset of a typed array view pointing to an ArrayBuffer. Set this to
+// 1 to enable a polyfill that works around the issue when it appears. This
+// bug is only relevant to WebGL 1, the affected browsers do not support WebGL 2.
+var WORKAROUND_OLD_WEBGL_UNIFORM_UPLOAD_IGNORED_OFFSET_BUG = 0;
+
 // Enables WebGL2 native functions. This mode will also create a WebGL2
 // context by default if no version is specified.
 var USE_WEBGL2 = 0;
@@ -421,11 +481,27 @@ var GL_PREINITIALIZED_CONTEXT = 0;
 // stbi_* functions directly yourself.
 var STB_IMAGE = 0;
 
-// Enable this to get support for non-modern browsers, node.js, etc. This gives you
+// If WORKAROUND_IOS_9_RIGHT_SHIFT_BUG==1, work around Safari/WebKit bug in iOS 9.3.5: https://bugs.webkit.org/show_bug.cgi?id=151514 where computing "a >> b" or "a >>> b" in
+// JavaScript would erroneously output 0 when a!=0 and b==0, after suitable JIT compiler optimizations have been applied to a function at runtime (bug does not
+// occur in debug builds). Fix was landed in https://trac.webkit.org/changeset/196591/webkit on Feb 15th 2016. iOS 9.3.5 was released on August 25 2016, but
+// oddly did not have the fix. iOS Safari 10.3.3 was released on July 19 2017, that no longer has the issue. Unknown which released version between these was the
+// first to contain the fix, though notable is that iOS 9.3.5 and iOS 10.3.3 are the two consecutive "end-of-life" versions of iOS that users are likely
+// to be on, e.g. iPhone 4s, iPad 2, iPad 3, iPad Mini 1, Pod Touch 5 all had end-of-life at iOS 9.3.5 (tested to be affected),
+// and iPad 4, iPhone 5 and iPhone 5c all had end-of-life at iOS 10.3.3 (confirmed not affected).
+// If you do not care about old iOS 9 support, keep this disabled.
+var WORKAROUND_IOS_9_RIGHT_SHIFT_BUG = 0;
+
+// If set, enables polyfilling for Math.clz32, Math.trunc, Math.imul, Math.fround.
+var POLYFILL_OLD_MATH_FUNCTIONS = 0;
+
+// Set this to enable compatibility emulations for old JavaScript engines. This gives you
 // the highest possible probability of the code working everywhere, even in rare old
 // browsers and shell environments. Specifically:
-//  * Add polyfilling for Math.clz32, Math.trunc, Math.imul, Math.fround.
-//  * Disable WebAssembly.
+//  * Add polyfilling for Math.clz32, Math.trunc, Math.imul, Math.fround. (-s POLYFILL_OLD_MATH_FUNCTIONS=1)
+//  * Work around iOS 9 right shift bug (-s WORKAROUND_IOS_9_RIGHT_SHIFT_BUG=1)
+//  * Work around old Chromium WebGL 1 bug (-s WORKAROUND_OLD_WEBGL_UNIFORM_UPLOAD_IGNORED_OFFSET_BUG=1)
+//  * Disable WebAssembly. (Must be paired with -s WASM=0)
+// You can also configure the above options individually.
 var LEGACY_VM_SUPPORT = 0;
 
 // By default, emscripten output will run on the web, in a web worker,
@@ -436,6 +512,8 @@ var LEGACY_VM_SUPPORT = 0;
 //    'worker' - a web worker environment.
 //    'node'   - Node.js.
 //    'shell'  - a JS shell like d8, js, or jsc.
+// Or it can be a comma-separated list of them, e.g., "web,worker". If this is
+// the empty string, then all runtime environments are supported.
 // (There is also a 'pthread' environment, see shell.js, but it cannot be specified
 // manually yet TODO)
 var ENVIRONMENT = '';
@@ -480,7 +558,7 @@ var EXCEPTION_CATCHING_WHITELIST = [];
 // or programs that don't need exit() to work.
 
 // For more explanations of this option, please visit
-// https://github.com/kripken/emscripten/wiki/Asyncify
+// https://github.com/emscripten-core/emscripten/wiki/Asyncify
 var NODEJS_CATCH_EXIT = 1;
 
 // Whether to enable asyncify transformation
@@ -524,16 +602,10 @@ var EXTRA_EXPORTED_RUNTIME_METHODS = [];
 var FS_LOG = 0;
 
 // If set to nonzero, the provided virtual filesystem if treated
-// case-insensitive, like Windows and OSX do. If set to 0, the VFS is
+// case-insensitive, like Windows and macOS do. If set to 0, the VFS is
 // case-sensitive, like on Linux.
 var CASE_INSENSITIVE_FS = 0;
 
-// If set to nonzero, MEMFS will always utilize typed arrays as the backing
-// store for appending data to files. The default behavior is to use typed
-// arrays for files when the file size doesn't change after initial creation,
-// and for files that do change size, use normal JS arrays instead.
-var MEMFS_APPEND_TO_TYPED_ARRAYS = 0;
-
 // If set to 0, does not build in any filesystem support. Useful if you are just
 // doing pure computation, but not reading files or using any streams (including
 // fprintf, and other stdio.h things) or anything related. The one exception is
@@ -611,7 +683,14 @@ var PROFILING_FUNCS = 0;
 // may be slightly misleading, as this is for any JS library element, and not
 // just functions. For example, you can include the Browser object by adding
 // "$Browser" to this list.
-var DEFAULT_LIBRARY_FUNCS_TO_INCLUDE = ['memcpy', 'memset', 'malloc', 'free'];
+var DEFAULT_LIBRARY_FUNCS_TO_INCLUDE = [
+	'memcpy',
+	'memset',
+	'malloc',
+	'free',
+	'emscripten_get_heap_size', // Used by dynamicAlloc() and -s FETCH=1
+	'emscripten_resize_heap' // Used by dynamicAlloc() and -s FETCH=1
+	];
 
 // This list is also used to determine auto-exporting of library dependencies
 // (i.e., functions that might be dependencies of JS library functions, that if
@@ -634,6 +713,7 @@ var SHELL_FILE = 0;
 
 // If set to 1, we emit relocatable code from the LLVM backend; both
 // globals and function pointers are all offset (by gb and fp, respectively)
+// Automatically set for SIDE_MODULE or MAIN_MODULE.
 var RELOCATABLE = 0;
 
 // A main module is a file compiled in a way that allows us to link it to
@@ -647,14 +727,10 @@ var MAIN_MODULE = 0;
 // Corresponds to MAIN_MODULE (also supports modes 1 and 2)
 var SIDE_MODULE = 0;
 
-// If this is a main module (MAIN_MODULE == 1), then
-// we will link these at runtime. They must have been built with
-// SIDE_MODULE == 1.
+// If this is a shared object (MAIN_MODULE == 1 || SIDE_MODULE == 1), then we
+// will link these at runtime. They must have been built with SIDE_MODULE == 1.
 var RUNTIME_LINKED_LIBS = [];
 
-// (deprecated option TODO: remove)
-var BUILD_AS_SHARED_LIB = 0;
-
 // If set to 1, this is a worker library, a special kind of library that is run
 // in a worker. See emscripten.h
 var BUILD_AS_WORKER = 0;
@@ -680,11 +756,8 @@ var PROXY_TO_PTHREAD = 0;
 // remove unused functions and globals, which might be used by another module we
 // are linked with.
 //
-// BUILD_AS_SHARED_LIB > 0 implies this, so it is only important to set this to
-// 1 when building the main file, and *if* that main file has symbols that the
-// library it will open will then access through an extern.  LINKABLE of 0 is
-// very useful in that we can reduce the size of the generated code very
-// significantly, by removing everything not actually used.
+// MAIN_MODULE and SIDE_MODULE both imply this, so it not normally necessary
+// to set this explicitly.
 var LINKABLE = 0;
 
 // Emscripten 'strict' build mode: Drop supporting any deprecated build options.
@@ -709,7 +782,7 @@ var WARN_ON_UNDEFINED_SYMBOLS = 1;
 // EXPORTED_FUNCTIONS will also be reported.
 var ERROR_ON_UNDEFINED_SYMBOLS = 1;
 
-// If set to 1, any -lfoo directives pointing to nonexisting library files will
+// If set to 1, any -lfoo directives pointing to nonexistent library files will
 // issue a linker error.
 
 // The default value for this is currently 0, but will be transitioned to 1 in
@@ -797,9 +870,19 @@ var MODULARIZE = 0;
 // EXPORT_NAME() to create the instance manually.
 //
 // Note that the promise-like API MODULARIZE provides isn't available here
-// (since you arean't creating the instance yourself).
+// (since you aren't creating the instance yourself).
 var MODULARIZE_INSTANCE = 0;
 
+// If we separate out asm.js with the --separate-asm option,
+// this is the name of the variable where the generated asm.js
+// Module is assigned to. This name can either be a property
+// of Module, or a freestanding variable name, like "var asmJs".
+// If you are XHRing in multiple asm.js built files, use this option to
+// assign the generated asm.js modules to different variable names
+// so that they do not conflict. Default name is 'Module["asm"]' if a custom
+// name is not passed in.
+var SEPARATE_ASM_MODULE_NAME = '';
+
 // Export using an ES6 Module export rather than a UMD export.  MODULARIZE must
 // be enabled for ES6 exports.
 var EXPORT_ES6 = 0;
@@ -913,23 +996,12 @@ var EMTERPRETIFY_ADVISE = 0;
 // and the advise mode will include them in its analysis.
 var EMTERPRETIFY_SYNCLIST = [];
 
-// If > 0, we split memory into chunks, of the size given in this parameter.
-//  * TOTAL_MEMORY becomes the maximum amount of memory, as chunks are allocated on
-//    demand. That means this achieves a result similar to ALLOW_MEMORY_GROWTH, but
-//    better since it can free chunks in the middle. You still to set
-//    ALLOW_MEMORY_GROWTH if you want memory to grow beyond the initial TOTAL_MEMORY
-//    target.
-//  * Larger SPLIT_MEMORY sizes are generally faster to run.
-// TODO: more docs
-// TODO: add malloc-split to embuilder
-var SPLIT_MEMORY = 0;
-
-// Similar to SAFE_HEAP, but for SPLIT_MEMORY.
-var SAFE_SPLIT_MEMORY = 0;
-
 // whether js opts will be run, after the main compiler
 var RUNNING_JS_OPTS = 0;
 
+// whether we are emitting JS glue code
+var EMITTING_JS = 1;
+
 // whether we are in the generate struct_info bootstrap phase
 var BOOTSTRAPPING_STRUCT_INFO = 0;
 
@@ -959,10 +1031,6 @@ var WASM_BACKEND = 0;
 // of using LLVM IR.
 var WASM_OBJECT_FILES = 0;
 
-// How we should run WebAssembly code. By default, we run it natively.
-// See binaryen's src/js/wasm.js-post.js for more details and options.
-var BINARYEN_METHOD = "native-wasm";
-
 // An optional comma-separated list of script hooks to run after binaryen,
 // in binaryen's /scripts dir.
 var BINARYEN_SCRIPTS = "";
@@ -1006,12 +1074,21 @@ var BINARYEN_ASYNC_COMPILATION = 1;
 // you can set it to override if you are a Binaryen developer.
 var BINARYEN_ROOT = "";
 
+// WebAssembly defines a "producers section" which compilers and tools can
+// annotate themselves in. Emscripten does not emit this by default, as it
+// increases code size, and some users may not want information about their tools
+// to be included in their builds for privacy or security reasons, see
+// https://github.com/WebAssembly/tool-conventions/issues/93.
+// TODO: currently this flag just controls whether we run the binaryen pass
+//       to strip it out from the wasm (where the LLVM wasm backend may have
+//       created it)
+var EMIT_PRODUCERS_SECTION = 0;
+
 // Whether to legalize the JS FFI interfaces (imports/exports) by wrapping them
 // to automatically demote i64 to i32 and promote f32 to f64. This is necessary
 // in order to interface with JavaScript, both for asm.js and wasm.  For
 // non-web/non-JS embeddings, setting this to 0 may be desirable.
-// LEGALIZE_JS_FFI=0 is incompatible with RUNNING_JS_OPTS and using non-wasm
-// BINARYEN_METHOD settings.
+// LEGALIZE_JS_FFI=0 is incompatible with RUNNING_JS_OPTS.
 var LEGALIZE_JS_FFI = 1;
 
 // Ports
@@ -1042,6 +1119,9 @@ var USE_ZLIB = 0;
 // 1 = use libpng from emscripten-ports
 var USE_LIBPNG = 0;
 
+// 1 = use Regal from emscripten-ports
+var USE_REGAL = 0;
+
 // 1 = use bullet from emscripten-ports
 var USE_BULLET = 0;
 
@@ -1054,6 +1134,10 @@ var USE_OGG = 0;
 // 1 = use freetype from emscripten-ports
 var USE_FREETYPE = 0;
 
+// Specify the SDL_mixer version that is being linked against.
+// Doesn't *have* to match USE_SDL, but a good idea.
+var USE_SDL_MIXER = 1;
+
 // 1 = use harfbuzz from harfbuzz upstream
 var USE_HARFBUZZ = 0;
 
@@ -1076,9 +1160,6 @@ var SDL2_IMAGE_FORMATS = [];
 //    legalizer
 var DEBUG_TAGS_SHOWING = [];
 
-// For internal use only
-var ORIGINAL_EXPORTED_FUNCTIONS = [];
-
 // The list of defines (C_DEFINES) was moved into struct_info.json in the same
 // directory.  That file is automatically parsed by tools/gen_struct_info.py.
 // If you modify the headers, just clear your cache and emscripten libc should
@@ -1177,6 +1258,8 @@ var BUNDLED_CD_DEBUG_FILE = "";
 
 // Is enabled, use the JavaScript TextDecoder API for string marshalling.
 // Enabled by default, set this to 0 to disable.
+// If set to 2, we assume TextDecoder is present and usable, and do not emit
+// any JS code to fall back if it is missing.
 var TEXTDECODER = 1;
 
 // Embind specific: If enabled, assume UTF-8 encoded data in std::string binding.
@@ -1227,28 +1310,116 @@ var ASMFS = 0;
 // then you can safely ignore this warning.
 var SINGLE_FILE = 0;
 
-// For internal use only (name of the file containing wasm text, if relevant).
+// if set to 1, then generated WASM files will contain a custom
+// "emscripten_metadata" section that contains information necessary
+// to execute the file without the accompanying JS file.
+var EMIT_EMSCRIPTEN_METADATA = 0;
+
+
+// Internal use only, from here
+
+// tracks the list of EM_ASM signatures that are proxied between threads.
+var PROXIED_FUNCTION_SIGNATURES = [];
+
+var ORIGINAL_EXPORTED_FUNCTIONS = [];
+
+// name of the file containing wasm text, if relevant
 var WASM_TEXT_FILE = '';
 
-// For internal use only (name of the file containing wasm binary, if relevant).
+// name of the file containing wasm binary, if relevant
 var WASM_BINARY_FILE = '';
 
-// For internal use only (name of the file containing asm.js, if relevant).
+// name of the file containing asm.js code, if relevant
 var ASMJS_CODE_FILE = '';
 
+// name of the file containing the pthread *.worker.js, if relevant
+var PTHREAD_WORKER_FILE = '';
+
 // Base URL the source mapfile, if relevant
 var SOURCE_MAP_BASE = '';
 
-// for internal use only
 var MEM_INIT_IN_WASM = 0;
 
 // If set to 1, src/base64Utils.js will be included in the bundle.
 // This is set internally when needed (SINGLE_FILE)
 var SUPPORT_BASE64_EMBEDDING = 0;
 
-// For internal use only, the possible environments the code may run in.
+// the possible environments the code may run in.
 var ENVIRONMENT_MAY_BE_WEB = 1;
 var ENVIRONMENT_MAY_BE_WORKER = 1;
 var ENVIRONMENT_MAY_BE_NODE = 1;
 var ENVIRONMENT_MAY_BE_SHELL = 1;
-var ENVIRONMENT_MAY_BE_WEB_OR_WORKER = 1;
+
+// passes information to emscripten.py about whether to minify
+// JS -> asm.js import names. Controlled by optimization level, enabled
+// at -O1 and higher, but disabled at -g2 and higher.
+var MINIFY_ASMJS_IMPORT_NAMES = 0;
+
+// the total static allocation, that is, how much to bump the start of memory
+// for static globals. received from the backend, and possibly increased due
+// to JS static allocations
+var STATIC_BUMP = -1;
+
+// the total initial wasm table size.
+var WASM_TABLE_SIZE = 0;
+
+// if set to 1, then generated WASM files will contain a custom
+// "emscripten_metadata" section that contains information necessary
+// to execute the file without the accompanying JS file.
+var EMIT_EMSCRIPTEN_METADATA = 0;
+
+// Tracks whether we are building with errno support enabled. Set to 0
+// to disable compiling errno support in altogether. This saves a little
+// bit of generated code size in applications that do not care about
+// POSIX errno variable. Setting this to 0 also requires using --closure
+// for effective code size optimizations to take place.
+var SUPPORT_ERRNO = 1;
+
+// Internal: An array of all symbols exported from asm.js/wasm module.
+var MODULE_EXPORTS = [];
+
+// Internal (testing only): Disables the blitOffscreenFramebuffer VAO path.
+var OFFSCREEN_FRAMEBUFFER_FORBID_VAO_PATH = 0;
+
+// Internal (testing only): Forces memory growing to fail.
+var TEST_MEMORY_GROWTH_FAILS = 0;
+
+// Advanced: Customize this array to reduce the set of asm.js runtime variables
+// that are generated. This allows minifying extra bit of asm.js code from unused
+// runtime code, if you know some of these are not needed.
+// (think of this as advanced manual DCE)
+var ASM_PRIMITIVE_VARS = ['__THREW__', 'threwValue', 'setjmpId', 'tempInt', 'tempBigInt', 'tempBigIntS', 'tempValue', 'tempDouble', 'tempFloat', 'tempDoublePtr', 'STACKTOP', 'STACK_MAX']
+
+// If true, uses minimal sized runtime without POSIX features, Module, preRun/preInit/etc.,
+// Emscripten built-in XHR loading or library_browser.js. Enable this setting to target
+// the smallest code size possible.
+// Set MINIMAL_RUNTIME=2 to further enable even more code size optimizations. These opts are
+// quite hacky, and work around limitations in Closure and other parts of the build system, so
+// they may not work in all generated programs (But can be useful for really small programs)
+var MINIMAL_RUNTIME = 0;
+
+// If building with MINIMAL_RUNTIME=1 and application uses sbrk()/malloc(), enable this. If you
+// are not using dynamic allocations, can set this to 0 to save code size. This setting is
+// ignored when building with -s MINIMAL_RUNTIME=0.
+var USES_DYNAMIC_ALLOC = 1;
+
+// Advanced manual dead code elimination:
+// Specifies the set of runtime JS functions that should be imported to the asm.js/wasm module.
+// Remove elements from this list to make build smaller if some of these are not needed.
+// In Wasm -O3/-Os builds, adjusting this is not necessary, as the Meta-DCE pass is able to
+// remove these, but if you are targeting asm.js or doing a -O2 build or lower, then this can
+// be beneficial.
+var RUNTIME_FUNCS_TO_IMPORT = ['abort', 'setTempRet0', 'getTempRet0']
+
+// Internal: stores the base name of the output file (-o TARGET_BASENAME.js)
+var TARGET_BASENAME = '';
+
+// If true, compiler supports setjmp() and longjmp(). If false, these APIs are not available.
+// If you are using C++ exceptions, but do not need setjmp()+longjmp() API, then you can set
+// this to 0 to save a little bit of code size and performance when catching exceptions.
+var SUPPORT_LONGJMP = 1;
+
+// If set to 1, disables old deprecated HTML5 API event target lookup behavior. When enabled,
+// there is no "Module.canvas" object, no magic "null" default handling, and DOM element
+// 'target' parameters are taken to refer to CSS selectors, instead of referring to DOM IDs.
+var DISABLE_DEPRECATED_FIND_EVENT_TARGET_BEHAVIOR = 0;
