@@ -5,17 +5,15 @@
 
 import os, shutil, logging
 
-TAG = 'version_55'
+TAG = 'version_67'
 
 def needed(settings, shared, ports):
-  if not settings.WASM: return False
-  try:
-    if shared.BINARYEN_ROOT: # if defined, and not falsey, we don't need the port
-      logging.debug('binaryen root already set to ' + shared.BINARYEN_ROOT)
-      settings.BINARYEN_ROOT = shared.BINARYEN_ROOT
-      return False
-  except:
-    pass
+  if not settings.WASM:
+    return False
+  if shared.BINARYEN_ROOT: # if defined, and not falsey, we don't need the port
+    logging.debug('binaryen root already set to ' + shared.BINARYEN_ROOT)
+    settings.BINARYEN_ROOT = shared.BINARYEN_ROOT
+    return False
   settings.BINARYEN_ROOT = os.path.join(ports.get_dir(), 'binaryen', 'binaryen-' + TAG)
   logging.debug('setting binaryen root to ' + settings.BINARYEN_ROOT)
   return True
