@@ -132,11 +132,12 @@ set = function() {}
 
 var settings_file = arguments_[0];
 var shell_file = arguments_[1];
+var process_macros = arguments_.indexOf('--expandMacros') >= 0;
 
 load(settings_file);
 load('parseTools.js');
 
 var from_html = read(shell_file);
-var to_html = preprocess(from_html);
+var to_html = process_macros ? processMacros(preprocess(from_html)) : preprocess(from_html);
 
 print(to_html);
